/*
 * Copyright notice from original mutt:
 * Copyright (C) 2000-5 Brendan Cully <brendan@kublai.com>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

/* remote host account manipulation (POP/IMAP) */

#include <lib-lib/lib-lib.h>
#include <lib-ui/enter.h>
#include <lib-ui/curses.h>

#include "mutt.h"
#include "account.h"

/* mutt_account_match: compare account info (host/port/user/login) */
int mutt_account_match (const ACCOUNT * a1, const ACCOUNT * a2)
{
    const char* user = NONULL (Username);
    const char* login = NONULL (Username);

    if (a1->type != a2->type)
        return 0;
    if (ascii_strcasecmp (a1->host, a2->host))
        return 0;
    if (a1->port != a2->port)
        return 0;

    if (a1->type == M_ACCT_TYPE_IMAP) {
        if (ImapUser && (ImapUser[0] != '\0'))
            user = ImapUser;
        if (ImapLogin && (ImapLogin[0] != '\0'))
            login = ImapLogin;
    }

    if (a1->type == M_ACCT_TYPE_POP && PopUser)
        user = PopUser;

#ifdef USE_NNTP
    if (a1->type == M_ACCT_TYPE_NNTP && NntpUser)
        user = NntpUser;
#endif

    if (a1->flags & a2->flags & M_ACCT_USER)
        return (!m_strcmp(a1->user, a2->user));
    if (a1->flags & M_ACCT_USER)
        return (!m_strcmp(a1->user, user));
    if (a2->flags & M_ACCT_USER)
        return (!m_strcmp(a2->user, user));

    return 1;
}

/* mutt_account_fromurl: fill account with information from url. */
int mutt_account_fromurl(ACCOUNT *account, ciss_url_t *url)
{
    /* must be present */
    if (!url->host)
        return -1;
    m_strcpy(account->host, sizeof(account->host), url->host);

    if (url->user) {
        m_strcpy(account->user, sizeof(account->user), url->user);
        account->flags |= M_ACCT_USER;
    }
    if (url->pass) {
        m_strcpy(account->pass, sizeof(account->pass), url->pass);
        account->flags |= M_ACCT_PASS;
    }
    if (url->port) {
        account->port = url->port;
        account->flags |= M_ACCT_PORT;
    }

    return 0;
}

/* mutt_account_tourl: fill URL with info from account. The URL information
 *   is a set of pointers into account - don't free or edit account until
 *   you've finished with url (make a copy of account if you need it for
 *   a while). */
void mutt_account_tourl (ACCOUNT * account, ciss_url_t * url)
{
    url->scheme = U_UNKNOWN;
    url->user = NULL;
    url->pass = NULL;
    url->port = 0;

    if (account->type == M_ACCT_TYPE_IMAP) {
        if (account->flags & M_ACCT_SSL)
            url->scheme = U_IMAPS;
        else
            url->scheme = U_IMAP;
    }

    if (account->type == M_ACCT_TYPE_POP) {
        if (account->flags & M_ACCT_SSL)
            url->scheme = U_POPS;
        else
            url->scheme = U_POP;
    }

#ifdef USE_NNTP
    if (account->type == M_ACCT_TYPE_NNTP) {
        if (account->flags & M_ACCT_SSL)
            url->scheme = U_NNTPS;
        else
            url->scheme = U_NNTP;
    }
#endif

    url->host = account->host;
    if (account->flags & M_ACCT_PORT)
        url->port = account->port;
    if (account->flags & M_ACCT_USER)
        url->user = account->user;
    if (account->flags & M_ACCT_PASS)
        url->pass = account->pass;
}

/* mutt_account_getuser: retrieve username into ACCOUNT, if necessary */
int mutt_account_getuser (ACCOUNT * account)
{
    char prompt[SHORT_STRING];

    /* already set */
    if (account->flags & M_ACCT_USER)
        return 0;
    else if ((account->type == M_ACCT_TYPE_IMAP) && !m_strisempty(ImapUser))
        m_strcpy(account->user, sizeof(account->user), ImapUser);
    else if ((account->type == M_ACCT_TYPE_POP) && !m_strisempty(PopUser))
        m_strcpy(account->user, sizeof(account->user), PopUser);
#ifdef USE_NNTP
    else if ((account->type == M_ACCT_TYPE_NNTP) && !m_strisempty(NntpUser))
        m_strcpy(account->user, sizeof(account->user), NntpUser);
#endif
    /* prompt (defaults to unix username), copy into account->user */
    else {
        snprintf(prompt, sizeof(prompt), _("Username at %s: "), account->host);
        m_strcpy(account->user, sizeof(account->user), NONULL(Username));
        if (mutt_get_field_unbuffered(prompt, account->user,
                                      sizeof(account->user), 0))
            return -1;
    }

    account->flags |= M_ACCT_USER;

    return 0;
}

int mutt_account_getlogin (ACCOUNT* account)
{
    /* already set */
    if (account->flags & M_ACCT_LOGIN)
        return 0;
    else if (account->type == M_ACCT_TYPE_IMAP)
    {
        if (!m_strisempty(ImapLogin)) {
            m_strcpy(account->login, sizeof(account->login), ImapLogin);
            account->flags |= M_ACCT_LOGIN;
        }
    }

    if (!(account->flags & M_ACCT_LOGIN)) {
        mutt_account_getuser (account);
        m_strcpy(account->login, sizeof(account->login), account->user);
    }

    account->flags |= M_ACCT_LOGIN;

    return 0;
}

/* mutt_account_getpass: fetch password into ACCOUNT, if neccessary */
int mutt_account_getpass (ACCOUNT * account)
{
    char prompt[SHORT_STRING];

    if (account->flags & M_ACCT_PASS)
        return 0;
    else if ((account->type == M_ACCT_TYPE_IMAP) && !m_strisempty(ImapPass))
        m_strcpy(account->pass, sizeof(account->pass), ImapPass);
    else if ((account->type == M_ACCT_TYPE_POP) && !m_strisempty(PopPass))
        m_strcpy(account->pass, sizeof(account->pass), PopPass);
#ifdef USE_NNTP
    else if ((account->type == M_ACCT_TYPE_NNTP) && !m_strisempty(NntpPass))
        m_strcpy(account->pass, sizeof(account->pass), NntpPass);
#endif
    else {
        snprintf(prompt, sizeof(prompt), _("Password for %s@%s: "),
                 account->flags & M_ACCT_LOGIN ? account->login : account->user,
                 account->host);
        account->pass[0] = '\0';
        if (mutt_get_field_unbuffered(prompt, account->pass,
                                      sizeof(account->pass), M_PASS))
            return -1;
    }

    account->flags |= M_ACCT_PASS;

    return 0;
}

void mutt_account_unsetpass (ACCOUNT * account)
{
    account->flags &= ~M_ACCT_PASS;
}
